@@ -9,17 +9,18 @@
 #include "mozilla/EventStateManager.h"
 #include "mozilla/PointerLockManager.h"
 #include "mozilla/PresShell.h"
-#include "mozilla/SVGUtils.h"
+#include "mozilla/StyleSheet.h"
 #include "mozilla/dom/AnimatableBinding.h"
 #include "mozilla/dom/Document.h"
 #include "mozilla/dom/HTMLInputElement.h"
 #include "mozilla/dom/ShadowRoot.h"
 #include "mozilla/dom/StyleSheetList.h"
 #include "nsTHashtable.h"
+#include "nsContentUtils.h"
 #include "nsFocusManager.h"
-#include "nsIRadioVisitor.h"
 #include "nsIFormControl.h"
 #include "nsLayoutUtils.h"
+#include "nsNameSpaceManager.h"
 #include "nsWindowSizes.h"
 
 namespace mozilla::dom {
@@ -55,7 +56,7 @@ void DocumentOrShadowRoot::AddSizeOfOwnedSheetArrayExcludingThis(
 
 void DocumentOrShadowRoot::AddSizeOfExcludingThis(nsWindowSizes& aSizes) const {
   AddSizeOfOwnedSheetArrayExcludingThis(aSizes, mStyleSheets);
-  aSizes.mDOMOtherSize +=
+  aSizes.mDOMSizes.mDOMOtherSize +=
       mIdentifierMap.SizeOfExcludingThis(aSizes.mState.mMallocSizeOf);
 }
 
@@ -91,6 +92,7 @@ void DocumentOrShadowRoot::RemoveStyleSheet(StyleSheet& aSheet) {
   mStyleSheets.RemoveElementAt(index);
   RemoveSheetFromStylesIfApplicable(*sheet);
   sheet->ClearAssociatedDocumentOrShadowRoot();
+  AsNode().OwnerDoc()->PostStyleSheetRemovedEvent(aSheet);
 }
 
 void DocumentOrShadowRoot::RemoveSheetFromStylesIfApplicable(
@@ -106,23 +108,26 @@ void DocumentOrShadowRoot::RemoveSheetFromStylesIfApplicable(
   }
 }
 
-// https://wicg.github.io/construct-stylesheets/#dom-documentorshadowroot-adoptedstylesheets
-void DocumentOrShadowRoot::SetAdoptedStyleSheets(
-    const Sequence<OwningNonNull<StyleSheet>>& aAdoptedStyleSheets,
-    ErrorResult& aRv) {
+// https://drafts.csswg.org/cssom/#dom-documentorshadowroot-adoptedstylesheets
+void DocumentOrShadowRoot::OnSetAdoptedStyleSheets(StyleSheet& aSheet,
+                                                   uint32_t aIndex,
+                                                   ErrorResult& aRv) {
   Document& doc = *AsNode().OwnerDoc();
-  for (const OwningNonNull<StyleSheet>& sheet : aAdoptedStyleSheets) {
-    // 2.1 Check if all sheets are constructed, else throw NotAllowedError
-    if (!sheet->IsConstructed()) {
+  // 1. If value’s constructed flag is not set, or its constructor document is
+  // not equal to this DocumentOrShadowRoot's node document, throw a
+  // "NotAllowedError" DOMException.
+
+  if (!StaticPrefs::
+          dom_webcomponents_lift_adoptedstylesheets_restriction_enabled()) {
+    if (!aSheet.IsConstructed()) {
       return aRv.ThrowNotAllowedError(
-          "Each adopted style sheet must be created through the Constructable "
+          "Adopted style sheet must be created through the Constructable "
           "StyleSheets API");
     }
-    // 2.2 Check if all sheets' constructor documents match the
-    // DocumentOrShadowRoot's node document, else throw NotAlloweError
-    if (!sheet->ConstructorDocumentMatches(doc)) {
+
+    if (!aSheet.ConstructorDocumentMatches(doc)) {
       return aRv.ThrowNotAllowedError(
-          "Each adopted style sheet's constructor document must match the "
+          "Adopted style sheet's constructor document must match the "
           "document or shadow root's node document");
     }
   }
@@ -130,84 +135,79 @@ void DocumentOrShadowRoot::SetAdoptedStyleSheets(
   auto* shadow = ShadowRoot::FromNode(AsNode());
   MOZ_ASSERT((mKind == Kind::ShadowRoot) == !!shadow);
 
-  StyleSheetSet set(aAdoptedStyleSheets.Length());
-  size_t commonPrefix = 0;
-
-  // Find the index at which the new array differs from the old array.
-  // We don't want to do extra work for the sheets that both arrays have.
-  size_t min =
-      std::min(aAdoptedStyleSheets.Length(), mAdoptedStyleSheets.Length());
-  for (size_t i = 0; i < min; ++i) {
-    if (aAdoptedStyleSheets[i] != mAdoptedStyleSheets[i]) {
-      break;
-    }
-    ++commonPrefix;
-    set.PutEntry(mAdoptedStyleSheets[i]);
+  auto existingIndex = mAdoptedStyleSheets.LastIndexOf(&aSheet);
+  // Ensure it's in the backing array at the right index.
+  mAdoptedStyleSheets.InsertElementAt(aIndex, &aSheet);
+  if (existingIndex == mAdoptedStyleSheets.NoIndex) {
+    // common case: we're not already adopting this sheet.
+    aSheet.AddAdopter(*this);
+  } else if (existingIndex < aIndex) {
+    // We're inserting an already-adopted stylesheet in a later position, so
+    // this one should take precedent and we should remove the old one.
+    RemoveSheetFromStylesIfApplicable(aSheet);
+  } else {
+    // The sheet is already at a position later than or equal to the current
+    // one, and is already adopted by us, we have nothing to do here other than
+    // adding to the current list.
+    return;
   }
 
-  // Try to truncate the sheets to a common prefix.
-  // If the prefix contains duplicates of sheets that we are removing,
-  // we are just going to re-build everything from scratch.
-  if (commonPrefix != mAdoptedStyleSheets.Length()) {
-    StyleSheetSet removedSet(mAdoptedStyleSheets.Length() - commonPrefix);
-    for (size_t i = mAdoptedStyleSheets.Length(); i != commonPrefix; --i) {
-      StyleSheet* sheetToRemove = mAdoptedStyleSheets.ElementAt(i - 1);
-      if (MOZ_UNLIKELY(set.Contains(sheetToRemove))) {
-        // Fixing duplicate sheets would require insertions/removals from the
-        // style set. We may as well just rebuild the whole thing from scratch.
-        set.Clear();
-        // Note that setting this to zero means we'll continue the loop until
-        // all the sheets are cleared.
-        commonPrefix = 0;
-      }
-      if (MOZ_LIKELY(removedSet.EnsureInserted(sheetToRemove))) {
-        RemoveSheetFromStylesIfApplicable(*sheetToRemove);
-        sheetToRemove->RemoveAdopter(*this);
-      }
-    }
-    mAdoptedStyleSheets.TruncateLength(commonPrefix);
-  }
-
-  // 3. Set the adopted style sheets to the new sheets
-  mAdoptedStyleSheets.SetCapacity(aAdoptedStyleSheets.Length());
-
-  // Only add sheets that are not already in the common prefix.
-  for (const auto& sheet : Span(aAdoptedStyleSheets).From(commonPrefix)) {
-    if (MOZ_UNLIKELY(!set.EnsureInserted(sheet))) {
-      // The idea is that this case is rare, so we pay the price of removing the
-      // old sheet from the styles and append it later rather than the other way
-      // around.
-      RemoveSheetFromStylesIfApplicable(*sheet);
+  if (aSheet.IsApplicable()) {
+    if (mKind == Kind::Document) {
+      doc.AddStyleSheetToStyleSets(aSheet);
     } else {
-      sheet->AddAdopter(*this);
+      shadow->InsertSheetIntoAuthorData(aIndex, aSheet, mAdoptedStyleSheets);
     }
-    mAdoptedStyleSheets.AppendElement(sheet);
-    if (sheet->IsApplicable()) {
-      if (mKind == Kind::Document) {
-        doc.AddStyleSheetToStyleSets(*sheet);
-      } else {
-        shadow->InsertSheetIntoAuthorData(mAdoptedStyleSheets.Length() - 1,
-                                          *sheet, mAdoptedStyleSheets);
-      }
+  }
+}
+
+void DocumentOrShadowRoot::OnDeleteAdoptedStyleSheets(StyleSheet& aSheet,
+                                                      uint32_t aIndex,
+                                                      ErrorResult&) {
+  MOZ_ASSERT(mAdoptedStyleSheets.ElementAt(aIndex) == &aSheet);
+  mAdoptedStyleSheets.RemoveElementAt(aIndex);
+  auto existingIndex = mAdoptedStyleSheets.LastIndexOf(&aSheet);
+  if (existingIndex != mAdoptedStyleSheets.NoIndex && existingIndex >= aIndex) {
+    // The sheet is still adopted by us and was already later from the one we're
+    // removing, so nothing to do.
+    return;
+  }
+
+  RemoveSheetFromStylesIfApplicable(aSheet);
+  if (existingIndex == mAdoptedStyleSheets.NoIndex) {
+    // The sheet is no longer adopted by us.
+    aSheet.RemoveAdopter(*this);
+  } else if (aSheet.IsApplicable()) {
+    // We need to re-insert the sheet at the right (pre-existing) index.
+    nsINode& node = AsNode();
+    if (mKind == Kind::Document) {
+      node.AsDocument()->AddStyleSheetToStyleSets(aSheet);
+    } else {
+      ShadowRoot::FromNode(node)->InsertSheetIntoAuthorData(
+          existingIndex, aSheet, mAdoptedStyleSheets);
     }
   }
 }
 
 void DocumentOrShadowRoot::ClearAdoptedStyleSheets() {
-  EnumerateUniqueAdoptedStyleSheetsBackToFront([&](StyleSheet& aSheet) {
-    RemoveSheetFromStylesIfApplicable(aSheet);
-    aSheet.RemoveAdopter(*this);
-  });
-  mAdoptedStyleSheets.Clear();
+  auto* shadow = ShadowRoot::FromNode(AsNode());
+  auto* doc = shadow ? nullptr : AsNode().AsDocument();
+  MOZ_ASSERT(shadow || doc);
+  IgnoredErrorResult rv;
+  while (!mAdoptedStyleSheets.IsEmpty()) {
+    if (shadow) {
+      ShadowRoot_Binding::AdoptedStyleSheetsHelpers::RemoveLastElement(shadow,
+                                                                       rv);
+    } else {
+      Document_Binding::AdoptedStyleSheetsHelpers::RemoveLastElement(doc, rv);
+    }
+    MOZ_DIAGNOSTIC_ASSERT(!rv.Failed(), "Removal doesn't fail");
+  }
 }
 
 void DocumentOrShadowRoot::CloneAdoptedSheetsFrom(
     const DocumentOrShadowRoot& aSource) {
-  if (!aSource.AdoptedSheetCount()) {
-    return;
-  }
-  Sequence<OwningNonNull<StyleSheet>> list;
-  if (!list.SetCapacity(mAdoptedStyleSheets.Length(), fallible)) {
+  if (aSource.mAdoptedStyleSheets.IsEmpty()) {
     return;
   }
 
@@ -217,33 +217,41 @@ void DocumentOrShadowRoot::CloneAdoptedSheetsFrom(
       sourceDoc.GetProperty(nsGkAtoms::adoptedsheetclones));
   MOZ_ASSERT(clonedSheetMap);
 
+  // We don't need to care about the reflector (AdoptedStyleSheetsHelpers and
+  // so) because this is only used for static documents.
   for (const StyleSheet* sheet : aSource.mAdoptedStyleSheets) {
-    RefPtr<StyleSheet> clone = clonedSheetMap->WithEntryHandle(
-        sheet, [&sheet, &ownerDoc](auto&& entry) {
-          return entry.OrInsertWith(
-              [&] { return sheet->CloneAdoptedSheet(ownerDoc); });
-        });
+    RefPtr<StyleSheet> clone = clonedSheetMap->LookupOrInsertWith(
+        sheet, [&] { return sheet->CloneAdoptedSheet(ownerDoc); });
     MOZ_ASSERT(clone);
     MOZ_DIAGNOSTIC_ASSERT(clone->ConstructorDocumentMatches(ownerDoc));
-    DebugOnly<bool> succeeded = list.AppendElement(std::move(clone), fallible);
-    MOZ_ASSERT(succeeded);
+    ErrorResult rv;
+    OnSetAdoptedStyleSheets(*clone, mAdoptedStyleSheets.Length(), rv);
+    MOZ_ASSERT(!rv.Failed());
   }
-
-  ErrorResult rv;
-  SetAdoptedStyleSheets(list, rv);
-  MOZ_ASSERT(!rv.Failed());
 }
 
-Element* DocumentOrShadowRoot::GetElementById(const nsAString& aElementId) {
+Element* DocumentOrShadowRoot::GetElementById(
+    const nsAString& aElementId) const {
   if (MOZ_UNLIKELY(aElementId.IsEmpty())) {
-    nsContentUtils::ReportEmptyGetElementByIdArg(AsNode().OwnerDoc());
+    ReportEmptyGetElementByIdArg();
     return nullptr;
   }
 
   if (IdentifierMapEntry* entry = mIdentifierMap.GetEntry(aElementId)) {
-    if (Element* el = entry->GetIdElement()) {
-      return el;
-    }
+    return entry->GetIdElement();
+  }
+
+  return nullptr;
+}
+
+Element* DocumentOrShadowRoot::GetElementById(nsAtom* aElementId) const {
+  if (MOZ_UNLIKELY(aElementId == nsGkAtoms::_empty)) {
+    ReportEmptyGetElementByIdArg();
+    return nullptr;
+  }
+
+  if (IdentifierMapEntry* entry = mIdentifierMap.GetEntry(aElementId)) {
+    return entry->GetIdElement();
   }
 
   return nullptr;
@@ -266,7 +274,7 @@ already_AddRefed<nsContentList> DocumentOrShadowRoot::GetElementsByTagNameNS(
   int32_t nameSpaceId = kNameSpaceID_Wildcard;
 
   if (!aNamespaceURI.EqualsLiteral("*")) {
-    aResult = nsContentUtils::NameSpaceManager()->RegisterNameSpace(
+    aResult = nsNameSpaceManager::GetInstance()->RegisterNameSpace(
         aNamespaceURI, nameSpaceId);
     if (aResult.Failed()) {
       return nullptr;
@@ -282,8 +290,8 @@ already_AddRefed<nsContentList> DocumentOrShadowRoot::GetElementsByClassName(
   return nsContentUtils::GetElementsByClassName(&AsNode(), aClasses);
 }
 
-nsIContent* DocumentOrShadowRoot::Retarget(nsIContent* aContent) const {
-  for (nsIContent* cur = aContent; cur; cur = cur->GetContainingShadowHost()) {
+nsINode* DocumentOrShadowRoot::Retarget(nsINode* aNode) const {
+  for (nsINode* cur = aNode; cur; cur = cur->GetContainingShadowHost()) {
     if (cur->SubtreeRoot() == &AsNode()) {
       return cur;
     }
@@ -296,7 +304,7 @@ Element* DocumentOrShadowRoot::GetRetargetedFocusedElement() {
   if (!content) {
     return nullptr;
   }
-  if (nsIContent* retarget = Retarget(content)) {
+  if (nsINode* retarget = Retarget(content)) {
     return retarget->AsElement();
   }
   return nullptr;
@@ -305,32 +313,18 @@ Element* DocumentOrShadowRoot::GetRetargetedFocusedElement() {
 Element* DocumentOrShadowRoot::GetPointerLockElement() {
   nsCOMPtr<Element> pointerLockedElement =
       PointerLockManager::GetLockedElement();
-  if (!pointerLockedElement) {
-    return nullptr;
-  }
-
-  nsIContent* retargetedPointerLockedElement = Retarget(pointerLockedElement);
-  return retargetedPointerLockedElement &&
-                 retargetedPointerLockedElement->IsElement()
-             ? retargetedPointerLockedElement->AsElement()
-             : nullptr;
+  return Element::FromNodeOrNull(Retarget(pointerLockedElement));
 }
 
-Element* DocumentOrShadowRoot::GetFullscreenElement() {
+Element* DocumentOrShadowRoot::GetFullscreenElement() const {
   if (!AsNode().IsInComposedDoc()) {
     return nullptr;
   }
 
-  Element* element = AsNode().OwnerDoc()->GetUnretargetedFullScreenElement();
-  NS_ASSERTION(!element || element->State().HasState(NS_EVENT_STATE_FULLSCREEN),
+  Element* element = AsNode().OwnerDoc()->GetUnretargetedFullscreenElement();
+  NS_ASSERTION(!element || element->State().HasState(ElementState::FULLSCREEN),
                "Fullscreen element should have fullscreen styles applied");
-
-  nsIContent* retargeted = Retarget(element);
-  if (retargeted && retargeted->IsElement()) {
-    return retargeted->AsElement();
-  }
-
-  return nullptr;
+  return Element::FromNodeOrNull(Retarget(element));
 }
 
 namespace {
@@ -350,17 +344,22 @@ enum class FlushLayout {
   Yes,
 };
 
+enum class PerformRetargeting {
+  No,
+  Yes,
+};
+
 template <typename NodeOrElement>
-NodeOrElement* CastTo(nsIContent* aContent);
+NodeOrElement* CastTo(nsINode*);
 
 template <>
-Element* CastTo<Element>(nsIContent* aContent) {
-  return aContent->AsElement();
+Element* CastTo<Element>(nsINode* aNode) {
+  return aNode->AsElement();
 }
 
 template <>
-nsINode* CastTo<nsINode>(nsIContent* aContent) {
-  return aContent;
+nsINode* CastTo<nsINode>(nsINode* aNode) {
+  return aNode;
 }
 
 template <typename NodeOrElement>
@@ -368,6 +367,7 @@ static void QueryNodesFromRect(DocumentOrShadowRoot& aRoot, const nsRect& aRect,
                                FrameForPointOptions aOptions,
                                FlushLayout aShouldFlushLayout,
                                Multiple aMultiple, ViewportType aViewportType,
+                               PerformRetargeting aPerformRetargeting,
                                nsTArray<RefPtr<NodeOrElement>>& aNodes) {
   static_assert(std::is_same<nsINode, NodeOrElement>::value ||
                     std::is_same<Element, NodeOrElement>::value,
@@ -375,6 +375,7 @@ static void QueryNodesFromRect(DocumentOrShadowRoot& aRoot, const nsRect& aRect,
 
   constexpr bool returningElements =
       std::is_same<Element, NodeOrElement>::value;
+  const bool retargeting = aPerformRetargeting == PerformRetargeting::Yes;
 
   nsCOMPtr<Document> doc = aRoot.AsNode().OwnerDoc();
 
@@ -403,35 +404,48 @@ static void QueryNodesFromRect(DocumentOrShadowRoot& aRoot, const nsRect& aRect,
                                   aOptions);
 
   for (nsIFrame* frame : frames) {
-    nsIContent* content = doc->GetContentInThisDocument(frame);
-    if (!content) {
+    nsINode* node = doc->GetContentInThisDocument(frame);
+    while (node && node->IsInNativeAnonymousSubtree()) {
+      nsIContent* root = node->GetClosestNativeAnonymousSubtreeRoot();
+      MOZ_ASSERT(root, "content is connected");
+      MOZ_ASSERT(root->IsRootOfNativeAnonymousSubtree(), "wat");
+      if (root == &aRoot.AsNode()) {
+        // If we're in the anonymous subtree root we care about, don't retarget.
+        break;
+      }
+      node = root->GetParentOrShadowHostNode();
+    }
+
+    if (!node) {
       continue;
     }
 
-    if (returningElements && !content->IsElement()) {
+    if (returningElements && !node->IsElement()) {
       // If this helper is called via ElementsFromPoint, we need to make sure
       // our frame is an element. Otherwise return whatever the top frame is
       // even if it isn't the top-painted element.
       // SVG 'text' element's SVGTextFrame doesn't respond to hit-testing, so
       // if 'content' is a child of such an element then we need to manually
       // defer to the parent here.
-      if (aMultiple == Multiple::Yes && !SVGUtils::IsInSVGTextSubtree(frame)) {
+      if (aMultiple == Multiple::Yes && !frame->IsInSVGTextSubtree()) {
         continue;
       }
 
-      content = content->GetParent();
-      if (ShadowRoot* shadow = ShadowRoot::FromNodeOrNull(content)) {
-        content = shadow->Host();
+      node = node->GetParent();
+      if (ShadowRoot* shadow = ShadowRoot::FromNodeOrNull(node)) {
+        node = shadow->Host();
       }
     }
 
     // XXXsmaug There is plenty of unspec'ed behavior here
     //         https://github.com/w3c/webcomponents/issues/735
     //         https://github.com/w3c/webcomponents/issues/736
-    content = aRoot.Retarget(content);
+    if (retargeting) {
+      node = aRoot.Retarget(node);
+    }
 
-    if (content && content != aNodes.SafeLastElement(nullptr)) {
-      aNodes.AppendElement(CastTo<NodeOrElement>(content));
+    if (node && node != aNodes.SafeLastElement(nullptr)) {
+      aNodes.AppendElement(CastTo<NodeOrElement>(node));
       if (aMultiple == Multiple::No) {
         return;
       }
@@ -444,6 +458,7 @@ static void QueryNodesFromPoint(DocumentOrShadowRoot& aRoot, float aX, float aY,
                                 FrameForPointOptions aOptions,
                                 FlushLayout aShouldFlushLayout,
                                 Multiple aMultiple, ViewportType aViewportType,
+                                PerformRetargeting aPerformRetargeting,
                                 nsTArray<RefPtr<NodeOrElement>>& aNodes) {
   // As per the spec, we return null if either coord is negative.
   if (!aOptions.mBits.contains(FrameForPointOption::IgnoreRootScrollFrame) &&
@@ -455,7 +470,8 @@ static void QueryNodesFromPoint(DocumentOrShadowRoot& aRoot, float aX, float aY,
   nscoord y = nsPresContext::CSSPixelsToAppUnits(aY);
   nsPoint pt(x, y);
   QueryNodesFromRect(aRoot, nsRect(pt, nsSize(1, 1)), aOptions,
-                     aShouldFlushLayout, aMultiple, aViewportType, aNodes);
+                     aShouldFlushLayout, aMultiple, aViewportType,
+                     aPerformRetargeting, aNodes);
 }
 
 }  // namespace
@@ -467,35 +483,37 @@ Element* DocumentOrShadowRoot::ElementFromPoint(float aX, float aY) {
 void DocumentOrShadowRoot::ElementsFromPoint(
     float aX, float aY, nsTArray<RefPtr<Element>>& aElements) {
   QueryNodesFromPoint(*this, aX, aY, {}, FlushLayout::Yes, Multiple::Yes,
-                      ViewportType::Layout, aElements);
+                      ViewportType::Layout, PerformRetargeting::Yes, aElements);
 }
 
 void DocumentOrShadowRoot::NodesFromPoint(float aX, float aY,
                                           nsTArray<RefPtr<nsINode>>& aNodes) {
   QueryNodesFromPoint(*this, aX, aY, {}, FlushLayout::Yes, Multiple::Yes,
-                      ViewportType::Layout, aNodes);
+                      ViewportType::Layout, PerformRetargeting::Yes, aNodes);
 }
 
 nsINode* DocumentOrShadowRoot::NodeFromPoint(float aX, float aY) {
   AutoTArray<RefPtr<nsINode>, 1> nodes;
   QueryNodesFromPoint(*this, aX, aY, {}, FlushLayout::Yes, Multiple::No,
-                      ViewportType::Layout, nodes);
+                      ViewportType::Layout, PerformRetargeting::Yes, nodes);
   return nodes.SafeElementAt(0);
 }
 
 Element* DocumentOrShadowRoot::ElementFromPointHelper(
     float aX, float aY, bool aIgnoreRootScrollFrame, bool aFlushLayout,
-    ViewportType aViewportType) {
+    ViewportType aViewportType, bool aPerformRetargeting) {
   EnumSet<FrameForPointOption> options;
   if (aIgnoreRootScrollFrame) {
     options += FrameForPointOption::IgnoreRootScrollFrame;
   }
 
   auto flush = aFlushLayout ? FlushLayout::Yes : FlushLayout::No;
+  auto performRetargeting =
+      aPerformRetargeting ? PerformRetargeting::Yes : PerformRetargeting::No;
 
   AutoTArray<RefPtr<Element>, 1> elements;
   QueryNodesFromPoint(*this, aX, aY, options, flush, Multiple::No,
-                      aViewportType, elements);
+                      aViewportType, performRetargeting, elements);
   return elements.SafeElementAt(0);
 }
 
@@ -530,7 +548,7 @@ void DocumentOrShadowRoot::NodesFromRect(float aX, float aY, float aTopSize,
 
   auto flush = aFlushLayout ? FlushLayout::Yes : FlushLayout::No;
   QueryNodesFromRect(*this, rect, options, flush, Multiple::Yes,
-                     ViewportType::Layout, aReturn);
+                     ViewportType::Layout, PerformRetargeting::No, aReturn);
 }
 
 Element* DocumentOrShadowRoot::AddIDTargetObserver(nsAtom* aID,
@@ -576,26 +594,10 @@ Element* DocumentOrShadowRoot::LookupImageElement(const nsAString& aId) {
   return entry ? entry->GetImageIdElement() : nullptr;
 }
 
-void DocumentOrShadowRoot::ReportEmptyGetElementByIdArg() {
+void DocumentOrShadowRoot::ReportEmptyGetElementByIdArg() const {
   nsContentUtils::ReportEmptyGetElementByIdArg(AsNode().OwnerDoc());
 }
 
-/**
- * A struct that holds all the information about a radio group.
- */
-struct nsRadioGroupStruct {
-  nsRadioGroupStruct()
-      : mRequiredRadioCount(0), mGroupSuffersFromValueMissing(false) {}
-
-  /**
-   * A strong pointer to the currently selected radio button.
-   */
-  RefPtr<HTMLInputElement> mSelectedRadioButton;
-  nsCOMArray<nsIFormControl> mRadioButtons;
-  uint32_t mRequiredRadioCount;
-  bool mGroupSuffersFromValueMissing;
-};
-
 void DocumentOrShadowRoot::GetAnimations(
     nsTArray<RefPtr<Animation>>& aAnimations) {
   // As with Element::GetAnimations we initially flush style here.
@@ -621,146 +623,6 @@ void DocumentOrShadowRoot::GetAnimations(
   aAnimations.Sort(AnimationPtrComparator<RefPtr<Animation>>());
 }
 
-nsresult DocumentOrShadowRoot::WalkRadioGroup(const nsAString& aName,
-                                              nsIRadioVisitor* aVisitor,
-                                              bool aFlushContent) {
-  nsRadioGroupStruct* radioGroup = GetOrCreateRadioGroup(aName);
-
-  for (int i = 0; i < radioGroup->mRadioButtons.Count(); i++) {
-    if (!aVisitor->Visit(radioGroup->mRadioButtons[i])) {
-      return NS_OK;
-    }
-  }
-
-  return NS_OK;
-}
-
-void DocumentOrShadowRoot::SetCurrentRadioButton(const nsAString& aName,
-                                                 HTMLInputElement* aRadio) {
-  nsRadioGroupStruct* radioGroup = GetOrCreateRadioGroup(aName);
-  radioGroup->mSelectedRadioButton = aRadio;
-}
-
-HTMLInputElement* DocumentOrShadowRoot::GetCurrentRadioButton(
-    const nsAString& aName) {
-  return GetOrCreateRadioGroup(aName)->mSelectedRadioButton;
-}
-
-nsresult DocumentOrShadowRoot::GetNextRadioButton(
-    const nsAString& aName, const bool aPrevious,
-    HTMLInputElement* aFocusedRadio, HTMLInputElement** aRadioOut) {
-  // XXX Can we combine the HTML radio button method impls of
-  //     Document and nsHTMLFormControl?
-  // XXX Why is HTML radio button stuff in Document, as
-  //     opposed to nsHTMLDocument?
-  *aRadioOut = nullptr;
-
-  nsRadioGroupStruct* radioGroup = GetOrCreateRadioGroup(aName);
-
-  // Return the radio button relative to the focused radio button.
-  // If no radio is focused, get the radio relative to the selected one.
-  RefPtr<HTMLInputElement> currentRadio;
-  if (aFocusedRadio) {
-    currentRadio = aFocusedRadio;
-  } else {
-    currentRadio = radioGroup->mSelectedRadioButton;
-    if (!currentRadio) {
-      return NS_ERROR_FAILURE;
-    }
-  }
-  int32_t index = radioGroup->mRadioButtons.IndexOf(currentRadio);
-  if (index < 0) {
-    return NS_ERROR_FAILURE;
-  }
-
-  int32_t numRadios = radioGroup->mRadioButtons.Count();
-  RefPtr<HTMLInputElement> radio;
-  do {
-    if (aPrevious) {
-      if (--index < 0) {
-        index = numRadios - 1;
-      }
-    } else if (++index >= numRadios) {
-      index = 0;
-    }
-    NS_ASSERTION(
-        static_cast<nsGenericHTMLFormElement*>(radioGroup->mRadioButtons[index])
-            ->IsHTMLElement(nsGkAtoms::input),
-        "mRadioButtons holding a non-radio button");
-    radio = static_cast<HTMLInputElement*>(radioGroup->mRadioButtons[index]);
-  } while (radio->Disabled() && radio != currentRadio);
-
-  radio.forget(aRadioOut);
-  return NS_OK;
-}
-
-void DocumentOrShadowRoot::AddToRadioGroup(const nsAString& aName,
-                                           HTMLInputElement* aRadio) {
-  nsRadioGroupStruct* radioGroup = GetOrCreateRadioGroup(aName);
-  radioGroup->mRadioButtons.AppendObject(aRadio);
-
-  if (aRadio->IsRequired()) {
-    radioGroup->mRequiredRadioCount++;
-  }
-}
-
-void DocumentOrShadowRoot::RemoveFromRadioGroup(const nsAString& aName,
-                                                HTMLInputElement* aRadio) {
-  nsRadioGroupStruct* radioGroup = GetOrCreateRadioGroup(aName);
-  radioGroup->mRadioButtons.RemoveObject(aRadio);
-
-  if (aRadio->IsRequired()) {
-    NS_ASSERTION(radioGroup->mRequiredRadioCount != 0,
-                 "mRequiredRadioCount about to wrap below 0!");
-    radioGroup->mRequiredRadioCount--;
-  }
-}
-
-uint32_t DocumentOrShadowRoot::GetRequiredRadioCount(
-    const nsAString& aName) const {
-  nsRadioGroupStruct* radioGroup = GetRadioGroup(aName);
-  return radioGroup ? radioGroup->mRequiredRadioCount : 0;
-}
-
-void DocumentOrShadowRoot::RadioRequiredWillChange(const nsAString& aName,
-                                                   bool aRequiredAdded) {
-  nsRadioGroupStruct* radioGroup = GetOrCreateRadioGroup(aName);
-
-  if (aRequiredAdded) {
-    radioGroup->mRequiredRadioCount++;
-  } else {
-    NS_ASSERTION(radioGroup->mRequiredRadioCount != 0,
-                 "mRequiredRadioCount about to wrap below 0!");
-    radioGroup->mRequiredRadioCount--;
-  }
-}
-
-bool DocumentOrShadowRoot::GetValueMissingState(const nsAString& aName) const {
-  nsRadioGroupStruct* radioGroup = GetRadioGroup(aName);
-  return radioGroup && radioGroup->mGroupSuffersFromValueMissing;
-}
-
-void DocumentOrShadowRoot::SetValueMissingState(const nsAString& aName,
-                                                bool aValue) {
-  nsRadioGroupStruct* radioGroup = GetOrCreateRadioGroup(aName);
-  radioGroup->mGroupSuffersFromValueMissing = aValue;
-}
-
-nsRadioGroupStruct* DocumentOrShadowRoot::GetRadioGroup(
-    const nsAString& aName) const {
-  nsRadioGroupStruct* radioGroup = nullptr;
-  mRadioGroups.Get(aName, &radioGroup);
-  return radioGroup;
-}
-
-nsRadioGroupStruct* DocumentOrShadowRoot::GetOrCreateRadioGroup(
-    const nsAString& aName) {
-  return mRadioGroups.WithEntryHandle(aName, [](auto&& entry) {
-    return entry.OrInsertWith([] { return MakeUnique<nsRadioGroupStruct>(); })
-        .get();
-  });
-}
-
 int32_t DocumentOrShadowRoot::StyleOrderIndexOfSheet(
     const StyleSheet& aSheet) const {
   if (aSheet.IsConstructed()) {
@@ -772,11 +634,6 @@ int32_t DocumentOrShadowRoot::StyleOrderIndexOfSheet(
   return mStyleSheets.IndexOf(&aSheet);
 }
 
-void DocumentOrShadowRoot::GetAdoptedStyleSheets(
-    nsTArray<RefPtr<StyleSheet>>& aAdoptedStyleSheets) const {
-  aAdoptedStyleSheets = mAdoptedStyleSheets.Clone();
-}
-
 void DocumentOrShadowRoot::TraverseSheetRefInStylesIfApplicable(
     StyleSheet& aSheet, nsCycleCollectionTraversalCallback& cb) {
   if (!aSheet.IsApplicable()) {
@@ -814,23 +671,9 @@ void DocumentOrShadowRoot::Traverse(DocumentOrShadowRoot* tmp,
   });
   NS_IMPL_CYCLE_COLLECTION_TRAVERSE(mAdoptedStyleSheets);
 
-  for (auto iter = tmp->mIdentifierMap.ConstIter(); !iter.Done(); iter.Next()) {
+  for (auto iter = tmp->mIdentifierMap.Iter(); !iter.Done(); iter.Next()) {
     iter.Get()->Traverse(&cb);
   }
-
-  for (auto iter = tmp->mRadioGroups.Iter(); !iter.Done(); iter.Next()) {
-    nsRadioGroupStruct* radioGroup = iter.UserData();
-    NS_CYCLE_COLLECTION_NOTE_EDGE_NAME(
-        cb, "mRadioGroups entry->mSelectedRadioButton");
-    cb.NoteXPCOMChild(ToSupports(radioGroup->mSelectedRadioButton));
-
-    uint32_t i, count = radioGroup->mRadioButtons.Count();
-    for (i = 0; i < count; ++i) {
-      NS_CYCLE_COLLECTION_NOTE_EDGE_NAME(
-          cb, "mRadioGroups entry->mRadioButtons[i]");
-      cb.NoteXPCOMChild(radioGroup->mRadioButtons[i]);
-    }
-  }
 }
 
 void DocumentOrShadowRoot::UnlinkStyleSheets(
@@ -852,7 +695,6 @@ void DocumentOrShadowRoot::Unlink(DocumentOrShadowRoot* tmp) {
   });
   NS_IMPL_CYCLE_COLLECTION_UNLINK(mAdoptedStyleSheets);
   tmp->mIdentifierMap.Clear();
-  tmp->mRadioGroups.Clear();
 }
 
 }  // namespace mozilla::dom
